#include "gd32f4xx.h"
#include "ptz_header_file.h"
#include "math.h"
#include "ptz_default_value.h"
#include "ptz_type_select.h"
#include "l6235d.h"
#include "service_presetbitscan.h"

#ifdef PTZ_BLDC_MOTOR

static BSP_OS_SEM ptz_hori_stop_mutex;//Դ
static BSP_OS_SEM ptz_vert_stop_mutex;
static char ptz_hori_stop_count;//ˮƽֹֹֹͣͣʱ
static char ptz_vert_stop_count;//ֱֹֹֹͣͣʱ

void ptz_sem_post_stop_mutex()//ͷ̨ͣԴ
{
  BSP_OS_SemPost(&ptz_hori_stop_mutex);
  BSP_OS_SemPost(&ptz_vert_stop_mutex);
}


void ptz_hori_start(char direction, float speed)
{
  BSP_OS_SemWait(&ptz_hori_stop_mutex, 0u);
  ptz_hori_pid_clear_zero();
  //趨תٶ
  g_ptz.hori_speed_set = speed;
  //趨ת
  g_ptz.hori_direction_last = g_ptz.hori_direction_set;
  g_ptz.hori_direction_set = direction;
  g_ptz.hori_direction_actual = g_ptz.hori_direction_set;
  //
  g_ptz.hori_start_stop_set = PTZ_HORI_START;
  g_ptz.hori_start_stop_actual = g_ptz.hori_start_stop_set;
  
#ifdef PTZ_MOTOR_CONTROL_L6235D
    l6235d_hori_set_direction(direction);
    l6235d_hori_start_stop(PTZ_HORI_START);
#endif  
  
  ptz_hori_stop_count = 0;
#ifdef PTZ_ELECTRIC_STABLE_L6235D
  g_ptz.hori_electric_stable.stable_stop_switch = 0;
#endif
  BSP_OS_SemPost(&ptz_hori_stop_mutex);
}


void ptz_hori_stop(unsigned short int time)
{
  BSP_OS_SemWait(&ptz_hori_stop_mutex, 0u);
  //ֹͣ
  g_ptz.hori_start_stop_set = PTZ_HORI_STOP;
  g_ptz.hori_start_stop_actual = g_ptz.hori_start_stop_set;
#ifdef PTZ_MOTOR_CONTROL_L6235D
    l6235d_hori_start_stop(PTZ_HORI_STOP);
#endif 
    
  //PID0
  ptz_hori_pid_clear_zero();
  //趨תٶ
  g_ptz.hori_speed_set = 0;
  g_ptz.hori_speed_actual = 0;
  
  if(ptz_hori_stop_count <= 0)
  {
    OSTimeDlyHMSM(0u, 0u, 0u, time);
    ptz_hori_stop_count = 0;
  }
  ptz_hori_stop_count ++;
  //ȶ
#ifdef PTZ_ELECTRIC_STABLE_L6235D
  ptz_hori_electric_stable_init();
#endif
  BSP_OS_SemPost(&ptz_hori_stop_mutex);
}


void ptz_vert_start(char direction, float speed)
{
  BSP_OS_SemWait(&ptz_vert_stop_mutex, 0u);
  ptz_vert_pid_clear_zero();
  //趨תٶ
  g_ptz.vert_speed_set = speed;
  //趨ת
  g_ptz.vert_direction_last = g_ptz.vert_direction_set;
  g_ptz.vert_direction_set = direction;
  g_ptz.vert_direction_actual = g_ptz.vert_direction_set;
  //
  g_ptz.vert_start_stop_set = PTZ_VERT_START;
  g_ptz.vert_start_stop_actual = g_ptz.vert_start_stop_set;
#ifdef PTZ_MOTOR_CONTROL_L6235D
    l6235d_vert_set_direction(direction);
    l6235d_vert_start_stop(PTZ_VERT_START);
#endif 
    
  ptz_vert_stop_count = 0;
#ifdef PTZ_ELECTRIC_STABLE_L6235D
  g_ptz.vert_electric_stable.stable_stop_switch = 0;
#endif
  BSP_OS_SemPost(&ptz_vert_stop_mutex);
}


void ptz_vert_stop(unsigned short int time)
{
  BSP_OS_SemWait(&ptz_vert_stop_mutex, 0u);
  if(g_ptz.vert_start_stop_set == PTZ_VERT_START)
  {
    //ֹͣ
    g_ptz.vert_start_stop_set = PTZ_VERT_STOP;
    g_ptz.vert_start_stop_actual = g_ptz.vert_start_stop_set;
    
#ifdef PTZ_MOTOR_CONTROL_L6235D
      l6235d_vert_start_stop(PTZ_VERT_STOP);
#endif
      
    //PID0
    ptz_vert_pid_clear_zero();
    //趨תٶ
    g_ptz.vert_speed_set = 0;
    g_ptz.vert_speed_actual = 0;
    
    if(ptz_vert_stop_count <= 0)
    {
      OSTimeDlyHMSM(0u, 0u, 0u, time);
      ptz_vert_stop_count = 0;
    }
    ptz_vert_stop_count ++;
    //̨ȶ
#ifdef PTZ_ELECTRIC_STABLE_L6235D
    ptz_vert_electric_stable_init();
#endif
  }
  
  BSP_OS_SemPost(&ptz_vert_stop_mutex);
}



/// ̨ˮƽת
static char ptz_hori_rotate_monitor_task()
{
  static float BNP_D;//ɲ뵱ǰλõľ
  static float BFP_D;//ɲԶ뵱ǰλõľ
  static unsigned int k;
  switch(g_ptz.hori_rotate_monitor_switch)
  {
    case 0://ر
      
      break;
      
    case PTZ_HORI_RIGHT_KEEP://תɲ
      //رת
      g_ptz.hori_rotate_monitor_switch = 0;
      //ж̨Ƿת
      if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
      {
        //жתǷͬ
        //תֱͬ޸õת
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_RIGHT)
        {
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
        }
        else//תͬ
        {
          //ɲ
          ptz_hori_stop(PTZ_HORI_STOP_TIME);
          //
          g_ptz.hori_direction_set = PTZ_HORI_DIR_RIGHT;
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          ptz_hori_start(PTZ_HORI_DIR_RIGHT, g_ptz.hori_speed_set);
        }
      }
      else//ûת
      {
        g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
        ptz_hori_start(PTZ_HORI_DIR_RIGHT, g_ptz.hori_speed_set);
      }
      break;
      
    case PTZ_HORI_LEFT_KEEP://תɲ
      //رת
      g_ptz.hori_rotate_monitor_switch = 0;
      //ж̨Ƿת
      if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
      {
        //жתǷͬ
        //תֱͬ޸õת
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_LEFT)
        {
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
        }
        else//תͬ
        {
          //ɲ
          ptz_hori_stop(PTZ_HORI_STOP_TIME);
          //
          g_ptz.hori_direction_set = PTZ_HORI_DIR_LEFT;
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          ptz_hori_start(PTZ_HORI_DIR_LEFT, g_ptz.hori_speed_set);
        }
      }
      else//ûת
      {
        g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
        ptz_hori_start(PTZ_HORI_DIR_LEFT, g_ptz.hori_speed_set);
      }
      break;
      
    case PTZ_HORI_GO_TO_ANGLE_A://Ƕȶλ
    case PTZ_HORI_MIN_DISTANCE:
    case PTZ_HORI_MAX_DISTANCE:
    case PTZ_HORI_RIGHT_ANGLE:
    case PTZ_HORI_LEFT_ANGLE:    
      //жǷҪת,תΪ0ͬһλҪת
      if(g_ptz.hori_rotate_plan.rotate_switch == 1)
      {
         g_ptz.hori_rotate_monitor_switch = PTZ_HORI_GO_TO_ANGLE_B;
        //Ǵͬһλ
        //жǷת
        //ת
        if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
        {
          //жתǷͬ
          //תͬ
          if(g_ptz.hori_direction_set == g_ptz.hori_rotate_plan.direction)
          {
            g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          }
          else//תֱͬ޸õת
          {
            //ɲ
            ptz_hori_stop(PTZ_HORI_STOP_TIME);
            //
            g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
            g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
            ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
          }
        }
        else//ûת
        {
          g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
        }
        
        //ɲСһ̶ȣСתת
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_LEFT)
        {
          if(g_ptz.hori_angle_actual >= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
               g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_rotate_plan.stop_near_angle) + g_ptz.hori_angle_actual) <=
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_RIGHT)
        {
          if(g_ptz.hori_angle_actual <= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_angle_actual) + g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
        
       
      }
      else//g_ptz.hori_rotate_plan.rotate_switch == 0;
      {
        g_ptz.hori_rotate_monitor_switch = 0;
        //Ҫתɲֹͣرռ
        g_ptz.hori_arrive_flag = 0;//תָλ
        ptz_location_return_return(LOCATION_HORI);//λش
        ptz_preset_bit_location_return_return();//Ԥλλش
        if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
        {
          ptz_hori_stop(PTZ_HORI_STOP_TIME);
        }
      }
      break;
    
      
    case PTZ_HORI_RIGHT_CYCLE:
    case PTZ_HORI_LEFT_CYCLE:
      if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
      {
        //жתǷͬ
        //תͬ
        if(g_ptz.hori_direction_set == g_ptz.hori_rotate_plan.direction)
        {
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
        }
        else//תֱͬ޸õת
        {
          //ɲ
          ptz_hori_stop(PTZ_HORI_STOP_TIME);
          //
          g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
        }
      }
      else//ûת
      {
        g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
        g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
        ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
      }
      g_ptz.hori_rotate_monitor_switch = PTZ_HORI_CYCLE;
      break;
      
    case PTZ_HORI_CYCLE:
      BNP_D = fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle);
      if(BNP_D > 360.0 / 2.0)
      {
        BNP_D = 360.0 - BNP_D;
      }
      BFP_D = fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_far_angle);
      if(BFP_D > 360.0 / 2.0)
      {
        BFP_D = 360.0 - BFP_D;
      }
      /*λ*/
      if(BNP_D + BFP_D > (g_ptz.hori_rotate_plan.stop_angle_range + 2.0))
      {
        g_ptz.hori_rotate_monitor_switch = PTZ_HORI_GO_TO_ANGLE_B;
      }
      break;
      
    case PTZ_HORI_GO_TO_ANGLE_B:
      BNP_D = fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle);
      if(BNP_D > 360.0 / 2.0)
      {
        BNP_D = 360.0 - BNP_D;
      }
      BFP_D = fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_far_angle);
      if(BFP_D > 360.0 / 2.0)
      {
        BFP_D = 360.0 - BFP_D;
      }
      /*λ*/
      if(BNP_D + BFP_D <= (g_ptz.hori_rotate_plan.stop_angle_range + 0.01))
      {
        if(g_ptz.hori_repeat_locate_switch == 0)//ظλر
        {
          g_ptz.hori_arrive_flag = 0;//תָλ
          ptz_location_return_return(LOCATION_HORI);//λش
          ptz_preset_bit_location_return_return();//Ԥλλش
        }
        g_ptz.hori_rotate_monitor_switch = 0;
        if(g_ptz.hori_repeat_locate_switch == 1)//ظλ
        {
          g_ptz.hori_rotate_monitor_switch = PTZ_HORI_REPEAT_LOCATE_A;
        }
        ptz_hori_stop(PTZ_HORI_STOP_TIME);
      }
      //ɲСһ̶ȣСתת
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_LEFT)
        {
          if(g_ptz.hori_angle_actual >= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
               g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_rotate_plan.stop_near_angle) + g_ptz.hori_angle_actual) <=
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_RIGHT)
        {
          if(g_ptz.hori_angle_actual <= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_angle_actual) + g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
      break;
      
    case PTZ_HORI_BRAKE://ֹͣת
      g_ptz.hori_rotate_monitor_switch = 0;
      ptz_hori_stop(PTZ_HORI_STOP_TIME);
      break;
    
    case PTZ_HORI_DEC_BRAKE_A://ɲ
      g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
      g_ptz.hori_rotate_monitor_switch = PTZ_HORI_DEC_BRAKE_B;
      k = 0;
      break;
      
    case PTZ_HORI_DEC_BRAKE_B:
       k++;//K1ʱ1ms
      if(g_ptz.hori_speed_actual <= PTZ_HORI_MIN_SPEED * 1.5 ||
         k >= 1400)
      {
        k = 0;
        g_ptz.hori_rotate_monitor_switch = 0;
        ptz_hori_stop(PTZ_HORI_STOP_TIME);
      }

      break;
    
    //ظλ·滮
    case  PTZ_HORI_REPEAT_LOCATE_A:
      ptz_hori_rotate_plan(PTZ_HORI_MIN_DISTANCE);
      g_ptz.hori_rotate_monitor_switch = PTZ_HORI_REPEAT_LOCATE_B;
      break;
    //ظλ
    case  PTZ_HORI_REPEAT_LOCATE_B:
      //жǷҪת,תΪ0ͬһλҪת
      if(g_ptz.hori_rotate_plan.rotate_switch == 1)
      {
        g_ptz.hori_rotate_monitor_switch = PTZ_HORI_REPEAT_LOCATE_C;
        //Ǵͬһλ
        //жǷת
        //ת
        if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
        {
          //жתǷͬ
          //תͬ
          if(g_ptz.hori_direction_set == g_ptz.hori_rotate_plan.direction)
          {
            g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          }
          else//תֱͬ޸õת
          {
            //ɲ
            ptz_hori_stop(PTZ_HORI_STOP_TIME);
            //
            g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
            g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
            ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
          }
        }
        else//ûת
        {
          g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
        }
        
        //ɲСһ̶ȣСתת
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_LEFT)
        {
          if(g_ptz.hori_angle_actual >= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
               g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_rotate_plan.stop_near_angle) + g_ptz.hori_angle_actual) <=
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_RIGHT)
        {
          if(g_ptz.hori_angle_actual <= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_angle_actual) + g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
        
      }
      else//g_ptz.hori_rotate_plan.rotate_switch == 0;
      {
        g_ptz.hori_rotate_monitor_switch = 0;
        //Ҫתɲֹͣرռ
        g_ptz.hori_arrive_flag = 0;//תָԤλ
        ptz_location_return_return(LOCATION_HORI);//λش
        ptz_preset_bit_location_return_return();//Ԥλλش
        if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
        {
          ptz_hori_stop(PTZ_HORI_STOP_TIME);
        }
      }
      break;
    //ظλжǷﵽλλ
    case  PTZ_HORI_REPEAT_LOCATE_C:
      BNP_D = fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle);
      if(BNP_D > 360.0 / 2.0)
      {
        BNP_D = 360.0 - BNP_D;
      }
      BFP_D = fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_far_angle);
      if(BFP_D > 360.0 / 2.0)
      {
        BFP_D = 360.0 - BFP_D;
      }
      /*λ*/
      if(BNP_D + BFP_D <= (g_ptz.hori_rotate_plan.stop_angle_range + 0.01))
      {
        g_ptz.hori_arrive_flag = 0;//תָλ
        ptz_location_return_return(LOCATION_HORI);//λش
        ptz_preset_bit_location_return_return();//Ԥλλش
        g_ptz.hori_rotate_monitor_switch = 0;
        ptz_hori_stop(PTZ_HORI_STOP_TIME);
      }
      //ɲСһ̶ȣСתת
      if(g_ptz.hori_direction_set == PTZ_HORI_DIR_LEFT)
      {
        if(g_ptz.hori_angle_actual >= g_ptz.hori_rotate_plan.stop_near_angle)
        {
          if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
             PTZ_HORI_MIN_SPEED_ANGLE)
          {
             g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
          }
        }
        else
        {
          if((fabs(360.0 - g_ptz.hori_rotate_plan.stop_near_angle) + g_ptz.hori_angle_actual) <=
             PTZ_HORI_MIN_SPEED_ANGLE)
          {
            g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
          }
        }
      }
      if(g_ptz.hori_direction_set == PTZ_HORI_DIR_RIGHT)
      {
        if(g_ptz.hori_angle_actual <= g_ptz.hori_rotate_plan.stop_near_angle)
        {
          if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
             PTZ_HORI_MIN_SPEED_ANGLE)
          {
            g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
          }
        }
        else
        {
          if((fabs(360.0 - g_ptz.hori_angle_actual) + g_ptz.hori_rotate_plan.stop_near_angle) <= 
             PTZ_HORI_MIN_SPEED_ANGLE)
          {
            g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
          }
        }
      }
      break;
      
    case PTZ_HORI_RIGHT_ANGLE_INC:
      //жǷҪת,תΪ0ͬһλҪת
      if(g_ptz.hori_rotate_plan.rotate_switch == 1)
      {
        g_ptz.hori_rotate_monitor_switch = PTZ_HORI_GO_TO_ANGLE_B;
        //Ǵͬһλ
        //жǷת
        //ת
        if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
        {
          //жתǷͬ
          //תͬ
          if(g_ptz.hori_direction_set == g_ptz.hori_rotate_plan.direction)
          {
            g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          }
          else//תֱͬ޸õת
          {
            //ɲ
            ptz_hori_stop(PTZ_HORI_STOP_TIME);
            //
            g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
            g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
            ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
          }
        }
        else//ûת
        {
          g_ptz.hori_direction_set = g_ptz.hori_rotate_plan.direction;
          g_ptz.hori_speed_set = g_ptz.hori_rotate_plan.max_speed;
          ptz_hori_start(g_ptz.hori_direction_set, g_ptz.hori_speed_set);
        }
        
        //ɲСһ̶ȣСתת
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_LEFT)
        {
          if(g_ptz.hori_angle_actual >= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
               g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_rotate_plan.stop_near_angle) + g_ptz.hori_angle_actual) <=
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
        if(g_ptz.hori_direction_set == PTZ_HORI_DIR_RIGHT)
        {
          if(g_ptz.hori_angle_actual <= g_ptz.hori_rotate_plan.stop_near_angle)
          {
            if(fabs(g_ptz.hori_angle_actual - g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
          else
          {
            if((fabs(360.0 - g_ptz.hori_angle_actual) + g_ptz.hori_rotate_plan.stop_near_angle) <= 
               PTZ_HORI_MIN_SPEED_ANGLE)
            {
              g_ptz.hori_speed_set = PTZ_HORI_MIN_SPEED;
            }
          }
        }
        
        
      }
      else//g_ptz.hori_rotate_plan.rotate_switch == 0;
      {
         g_ptz.hori_rotate_monitor_switch = 0;
        //Ҫתɲֹͣرռ
        g_ptz.hori_arrive_flag = 0;//תָλ
        ptz_location_return_return(LOCATION_HORI);//λش
        ptz_preset_bit_location_return_return();//Ԥλλش
        if(g_ptz.hori_start_stop_set == PTZ_HORI_START)
        {
          ptz_hori_stop(PTZ_HORI_STOP_TIME);
        }
      }
      break;
      
    case PTZ_HORI_LEFT_ANGLE_INC:
      
      break;
    
  }
  return 1;
}


/// ֱ̨ת
static char ptz_vert_rotate_monitor_task()
{
  static float BNP_D;//ɲ뵱ǰλõľ
  static float BFP_D;//ɲԶ뵱ǰλõľ
  static char cmd_type;
  static unsigned int k;
  switch(g_ptz.vert_rotate_monitor_switch)
  {
    case 0://ر
      
      break;
      
    case PTZ_VERT_GO_TO_ANGLE_A://תָǶ
    case PTZ_VERT_ANGLE:
    case PTZ_VERT_UP_KEEP:
    case PTZ_VERT_DOWN_KEEP:
      cmd_type = g_ptz.vert_rotate_monitor_switch;
      g_ptz.vert_rotate_monitor_switch = PTZ_VERT_GO_TO_ANGLE_B;
      if(g_ptz.vert_rotate_plan.rotate_switch == 1)
      {
        //жǷת
        //ת
        if(g_ptz.vert_start_stop_set == PTZ_VERT_START)
        {
          //жתǷͬ
          //תͬ
          if(g_ptz.vert_direction_set == g_ptz.vert_rotate_plan.direction)
          {
            g_ptz.vert_speed_set = g_ptz.vert_rotate_plan.max_speed;
          }
          else//תֱͬ޸õת
          {
            //ɲ
            ptz_vert_stop(PTZ_VERT_STOP_TIME);
            //
            g_ptz.vert_direction_set = g_ptz.vert_rotate_plan.direction;
            g_ptz.vert_speed_set = g_ptz.vert_rotate_plan.max_speed;
            ptz_vert_start(g_ptz.vert_direction_set, g_ptz.vert_speed_set);
          }
        }
        else//ûת
        {
          g_ptz.vert_direction_set = g_ptz.vert_rotate_plan.direction;
          g_ptz.vert_speed_set = g_ptz.vert_rotate_plan.max_speed;
          ptz_vert_start(g_ptz.vert_direction_set, g_ptz.vert_speed_set);
        }
        //ɲСһ̶ȣСתת
        if(fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_near_angle) <=
           PTZ_VERT_MIN_SPEED_ANGLE)
        {
          g_ptz.vert_speed_set = PTZ_VERT_MIN_SPEED;
        }
        
      }
      else//g_ptz.vert_rotate_plan.rotate_switch == 0
      {
        //ɲرת
        g_ptz.vert_arrive_flag = 0;//תָԤλ
        ptz_location_return_return(LOCATION_VERT);//λش
        ptz_preset_bit_location_return_return();//Ԥλλش
        g_ptz.vert_rotate_monitor_switch = 0;
        if(g_ptz.vert_start_stop_set == PTZ_VERT_START)
        {
          ptz_vert_stop(PTZ_VERT_STOP_TIME);
        }
      }
      break;
      
    case PTZ_VERT_GO_TO_ANGLE_B://תָǶȶλ
      BNP_D = fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_near_angle);
      BFP_D = fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_far_angle);
      /*λ*/
      if(BNP_D + BFP_D <= (g_ptz.vert_rotate_plan.stop_angle_range + 0.01))
      {
        if(g_ptz.vert_repeat_locate_switch == 0 &&
           cmd_type != PTZ_VERT_UP_KEEP &&
           cmd_type != PTZ_VERT_DOWN_KEEP)
        {
          g_ptz.vert_arrive_flag = 0;//תָԤλ
          ptz_location_return_return(LOCATION_VERT);//λش
          ptz_preset_bit_location_return_return();//Ԥλλش
        }
        g_ptz.vert_rotate_monitor_switch = 0;
        if(g_ptz.vert_repeat_locate_switch == 1 &&
           cmd_type != PTZ_VERT_UP_KEEP &&
           cmd_type != PTZ_VERT_DOWN_KEEP)
        {
          g_ptz.vert_rotate_monitor_switch = PTZ_VERT_REPEAT_LOCATE_A;
        }
        ptz_vert_stop(PTZ_VERT_STOP_TIME);
      }
      //ɲСһ̶ȣСתת
      if(fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_near_angle) <=
         PTZ_VERT_MIN_SPEED_ANGLE)
      {
        g_ptz.vert_speed_set = PTZ_VERT_MIN_SPEED;
      }
      break;
      
    case PTZ_VERT_BRAKE://ɲ
      g_ptz.vert_rotate_monitor_switch = 0;
      ptz_vert_stop(PTZ_VERT_STOP_TIME);
      break;
      
    case PTZ_VERT_DEC_BRAKE_A://ֱɲ
       g_ptz.vert_speed_set = PTZ_VERT_MIN_SPEED;
       g_ptz.vert_rotate_monitor_switch = PTZ_VERT_DEC_BRAKE_B;
       k = 0;
      break;
      
    case PTZ_VERT_DEC_BRAKE_B: 
      k ++;//K1,ʱ1ms 
      if(g_ptz.vert_speed_actual <= PTZ_VERT_MIN_SPEED * 1.5 ||
         k >= 1400)
      {
        k = 0;
        g_ptz.vert_rotate_monitor_switch = 0;
        ptz_vert_stop(PTZ_VERT_STOP_TIME);
      }
      break;
      
     //ظλ·滮
    case PTZ_VERT_REPEAT_LOCATE_A:
      ptz_vert_rotate_plan(PTZ_VERT_ANGLE);
      g_ptz.vert_rotate_monitor_switch = PTZ_VERT_REPEAT_LOCATE_B;
      break;
    //ظλת  
    case PTZ_VERT_REPEAT_LOCATE_B:
      if(g_ptz.vert_rotate_plan.rotate_switch == 1)
      {
        g_ptz.vert_rotate_monitor_switch = PTZ_VERT_REPEAT_LOCATE_C;
        //жǷת
        //ת
        if(g_ptz.vert_start_stop_set == PTZ_VERT_START)
        {
          //жתǷͬ
          //תͬ
          if(g_ptz.vert_direction_set == g_ptz.vert_rotate_plan.direction)
          {
            g_ptz.vert_speed_set = g_ptz.vert_rotate_plan.max_speed;
          }
          else//תֱͬ޸õת
          {
            //ɲ
            ptz_vert_stop(PTZ_VERT_STOP_TIME);
            //
            g_ptz.vert_direction_set = g_ptz.vert_rotate_plan.direction;
            g_ptz.vert_speed_set = g_ptz.vert_rotate_plan.max_speed;
            ptz_vert_start(g_ptz.vert_direction_set, g_ptz.vert_speed_set);
          }
        }
        else//ûת
        {
          g_ptz.vert_direction_set = g_ptz.vert_rotate_plan.direction;
          g_ptz.vert_speed_set = g_ptz.vert_rotate_plan.max_speed;
          ptz_vert_start(g_ptz.vert_direction_set, g_ptz.vert_speed_set);
        }
        //ɲСһ̶ȣСתת
        if(fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_near_angle) <=
           PTZ_VERT_MIN_SPEED_ANGLE)
        {
          g_ptz.vert_speed_set = PTZ_VERT_MIN_SPEED;
        }
        
      }
      else//g_ptz.vert_rotate_plan.rotate_switch == 0
      {
        //ɲرת
        g_ptz.vert_arrive_flag = 0;//תָλ
        ptz_location_return_return(LOCATION_VERT);//λش
        ptz_preset_bit_location_return_return();//Ԥλλش
        g_ptz.vert_rotate_monitor_switch = 0;
        if(g_ptz.vert_start_stop_set == PTZ_VERT_START)
        {
          ptz_vert_stop(PTZ_VERT_STOP_TIME);
        }
      }
      break;
      
    case PTZ_VERT_REPEAT_LOCATE_C:
      BNP_D = fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_near_angle);
      BFP_D = fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_far_angle);
      /*λ*/
      if(BNP_D + BFP_D <= (g_ptz.vert_rotate_plan.stop_angle_range + 0.01))
      {
        g_ptz.vert_arrive_flag = 0;//Ԥλɨ裬תָԤλ
        ptz_location_return_return(LOCATION_VERT);//λش
        ptz_preset_bit_location_return_return();//Ԥλλش
        g_ptz.vert_rotate_monitor_switch = 0;
        ptz_vert_stop(PTZ_VERT_STOP_TIME);
      }
      //ɲСһ̶ȣСתת
      if(fabs(g_ptz.vert_angle_actual - g_ptz.vert_rotate_plan.stop_near_angle) <=
         PTZ_VERT_MIN_SPEED_ANGLE)
      {
        g_ptz.vert_speed_set = PTZ_VERT_MIN_SPEED;
      }
      break;
  }

  //ֱ̨ڼλʱֱֹǶԽ
#ifdef PTZ_PHOTOELECTRIC_SWITCH
  if(g_ptz.vert_ps_sw1_state == PS_COVER)//¸λ
  {
    if(g_ptz.vert_start_stop_set == PTZ_VERT_START 
       && g_ptz.vert_direction_set == PTZ_VERT_DIR_DOWN)
    {
      g_ptz.vert_arrive_flag = 0;
      g_ptz.vert_rotate_monitor_switch = 0;
      ptz_vert_stop(PTZ_VERT_STOP_TIME);
    }
  }
  if(g_ptz.vert_ps_sw2_state == PS_COVER)//λ
  {
    if(g_ptz.vert_start_stop_set == PTZ_VERT_START 
       && g_ptz.vert_direction_set == PTZ_VERT_DIR_UP)
    {
      g_ptz.vert_arrive_flag = 0;
      g_ptz.vert_rotate_monitor_switch = 0;
      ptz_vert_stop(PTZ_VERT_STOP_TIME);
    }
  }
#endif
  
  return 1;
}

static void ptz_hori_rotate_task()
{
  while(1)
  {
    ptz_hori_rotate_monitor_task();
    OSTimeDlyHMSM(0u, 0u, 0u, 1u);
  }
}


static void ptz_vert_rotate_task()
{
  while(1)
  {
    ptz_vert_rotate_monitor_task();
    OSTimeDlyHMSM(0u, 0u, 0u, 1u);
  }
}

static OS_STK  task_hori_rotate_stk[TASK_HORI_ROATE_MONITOR_STK_SIZE];
static  void  creat_task_hori_rotate(void)
{ 
   CPU_INT08U  task_err;   
   CPU_INT08U  name_err;
   
   task_err = OSTaskCreateExt((void (*)(void *)) ptz_hori_rotate_task,           
		   (void           *) 0,
		   (OS_STK         *)&task_hori_rotate_stk[TASK_HORI_ROATE_MONITOR_STK_SIZE - 1],
		   (INT8U           ) TASK_HORI_ROATE_MONITOR_PRIO,                   
		   (INT16U          ) TASK_HORI_ROATE_MONITOR_PRIO,
		   (OS_STK         *)&task_hori_rotate_stk[0],
		   (INT32U          ) TASK_HORI_ROATE_MONITOR_STK_SIZE,
		   (void           *) 0,
		   (INT16U          )(OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR));     
    #if (OS_TASK_NAME_EN > 0)
    OSTaskNameSet(TASK_HORI_ROATE_MONITOR_PRIO, "ptz_hori_rotate_task", &name_err);
    #endif
       
}



static OS_STK  task_vert_rotate_stk[TASK_VERT_ROATE_MONITOR_STK_SIZE];
static  void  creat_task_vert_rotate(void)
{ 
   CPU_INT08U  task_err;   
   CPU_INT08U  name_err;
   
   task_err = OSTaskCreateExt((void (*)(void *)) ptz_vert_rotate_task,           
		   (void           *) 0,
		   (OS_STK         *)&task_vert_rotate_stk[TASK_VERT_ROATE_MONITOR_STK_SIZE - 1],
		   (INT8U           ) TASK_VERT_ROATE_MONITOR_PRIO,                   
		   (INT16U          ) TASK_VERT_ROATE_MONITOR_PRIO,
		   (OS_STK         *)&task_vert_rotate_stk[0],
		   (INT32U          ) TASK_VERT_ROATE_MONITOR_STK_SIZE,
		   (void           *) 0,
		   (INT16U          )(OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR));     
    #if (OS_TASK_NAME_EN > 0)
    OSTaskNameSet(TASK_VERT_ROATE_MONITOR_PRIO, "ptz_vert_rotate_task", &name_err);
    #endif
    
}


void init_rotate_monitor_module(void)
{
  BSP_OS_SemCreate(&ptz_hori_stop_mutex,1u,"ptz_hori_stop_mutex");
  BSP_OS_SemCreate(&ptz_vert_stop_mutex,1u,"ptz_vert_stop_mutex");

  l6235d_init();
  creat_task_hori_rotate();
  creat_task_vert_rotate();
  
}

#endif










